/*
 * blink_main.c
 *
 *  Created on: 2021/03/23
 *      Author: 0170886
 */

#include <kernel.h>
#include "kernel_cfg.h"
#include <t_syslog.h>
#include <t_stdlib.h>

#include "cup_timer.h"
#include "common.h"
#include "blink.h"

/* 外部参照関数宣言 */
extern void StIdleEvTimeOut(void);
extern void StIdleEvAvtive(void);
extern void StLedOffEvOff(void);
extern void StLedOnEvOff(void);
extern void StLedOffEvBlink(void);
extern void StLedOnEvBlink(void);

/*
 *  Glboal Variable
 */
TBL_CTL		gbl_ctl;

/*
 *  Initialization Routine
 */
void
blink_task_init(intptr_t exinf){
	GBL_CTLF = 0;
	GBL_BCOUNT = 0;
	GBL_NOWSTS = ST_BLINK_IDLE;
	GBL_NEXTSTS = ST_BLINK_IDLE;
}

/* ブリンクタスクのジャンプテーブルの定義 */
static const CALLBACK_BL BLStateTable[ST_BLINK_NUM][EV_BLINK_NUM]={
/*   TIMEOUT通知,       ACTIVE通知,      OFF通知,          0.25秒経過 */
	{&StIdleEvTimeOut,  &StIdleEvAvtive, NULL,			   NULL				},/* 休止状態 		*/
    {NULL,				NULL,			 &StLedOffEvOff,   &StLedOffEvBlink	},/* 点滅状態(消灯)	*/
    {NULL,				NULL,			 &StLedOnEvOff,    &StLedOnEvBlink	},/* 点滅状態(点灯)	*/
};

//  他タスクから通知されたイベント番号が範囲内かをチェックする関数
static bool_t
check_blink_event(void)
{
	bool_t ret;

	if(EV_BLINK_NUM <= GBL_RCVEV) {		/* uint32_tの為、マイナスは無い */
		ret = false;
	} else {
		ret = true;
	}
	return(ret);
}

void
blink_task(intptr_t exinf)
{
	const CALLBACK_BL *pFunc;

	for (;;) {
        SVC_PERROR(rcv_dtq(BLINK_DTQ, (intptr_t *)&GBL_RCVAD));
        if(true == check_blink_event()) {
        	pFunc = &BLStateTable[GBL_NOWSTS][GBL_RCVEV];
        	if(NULL != *pFunc){
        		(*pFunc)();		/* 現状態とイベントに対応する関数を実行 */
        	} else {
				DEBUG_SYSLOG(LOG_NOTICE, " ignore event. event id = %d ",GBL_RCVEV, " ");
        	}
        	if(GBL_CTLF & DBL_STSCHG) {		/* 状態遷移要求有り? */
        		GBL_CTLF &= ~DBL_STSCHG;	/* 要求クリア		 */
        		GBL_NOWSTS = GBL_NEXTSTS;   /* 状態更新 		 */
        	}
        }
        DEBUG_SYSLOG(LOG_NOTICE, " rel_mpf = 0x%p", GBL_RCVAD, " ");
		SVC_PERROR(rel_mpf(EVENT_MPF, GBL_RCVAD));
	}
}
